/******************************************************************************

Demonstrations of the 16 bit Experimenter Microcontroller




RGB LED Control

The Experimenter Kit supports an RGB LED that can produce nearly any color
by separately setting the red, green, and blue components of the color.  Each
color is controlled by a separate PWM on the PIC24F, which is configured using
Peripheral Pin Select to connect the proper pins to the appropriate PWM. The
pins are also configured to be open drain, enabling the complete range of
PWM control.

Select the color to modify by using the up and down touchpads.  Increase or
decrease the color intensity by using the right or left touchpads.  Exit the
demo by touching the center touchpad.

Saturation values are typically given in the range of 0x00-0xFF, so the PWM
timer and the sliders for each color are set to this range.  To interface with
the PWM module, we made two minor adjustments.  First, we cannot set the data
register OCxRS to 0, because that will generate a 50% duty cycle pulse at half
the desired frequency.  So the maximum saturation value we can set is 0xFE.
Alternately, the code can be modified to turn off the PWM and pull the output
low at the maximum saturation, and set the maximum saturation to 0xFF.
Second, to fully turn off the PWM when the saturation is 0, we have to set
OCxRS to 0x100 rather than the maximum saturation value; otherwise, we still
get a small pulse.


*******************************************************************************/
#include "p24fj64ga002.h"
/******************************************************************************

* File Name:       Demos.c
* Dependencies:    None
* Processor:       PIC24FJ64GA002
* Compiler:        C30 v3.02c
* Company:         Microchip Technology, Inc.


Author          Date    Comments
--------------------------------------------------------------------------------
KO          14-Feb-2008 First release
thk 3/19/09 major rework

*******************************************************************************/
//DOM-IGNORE-END

void RGBMapColorPins( void );
void RGBSetBlue( int saturation );
void RGBSetGreen( int saturation );
void RGBSetRed( int saturation );
void RGBTurnOffLED( void );
void RGBTurnOnLED( int initialRed, int initialGreen, int initialBlue );

//******************************************************************************
//******************************************************************************
// Constants
//******************************************************************************
//******************************************************************************

// To control the color, write the inverse of the saturation value to the PWM.
#define CONVERT_TO_COLOR(x)         (~x & 0xFF)
//#define CONVERT_TO_COLOR(x)         (x & 0xFF)

#define INITIAL_BLUE                (SLIDER_COLOR_MAX/2)    // Initial blue saturation.  Do not set to 0.
#define INITIAL_GREEN               (SLIDER_COLOR_MAX/2)    // Initial green saturation.  Do not set to 0.
#define INITIAL_RED                 (SLIDER_COLOR_MAX/2)    // Initial red saturation.  Do not set to 0.


#define SLIDER_COLOR_MAX            0xFE
#define SLIDER_COLOR_MIN            0x0000


#define SLIDER_RED                  0
#define SLIDER_GREEN                1
#define SLIDER_BLUE                 2


#define TRIS_INPUT                  1
#define TRIS_OUTPUT                 0

#define RGB_SATURATION_MINIMUM          0
#define RGB_SATURATION_MAXIMUM          0xFE    // See comments in Demo.c

#define SLIDER_COLOR_MAX            0xFE
#define PWM_CONFIGURATION_1         0x0006
#define PWM_CONFIGURATION_2         0x000C
#define PWM_OFF                     0x0000
//#define ID_RGB_RED                  80
#define ID_RGB_RED_SLIDER           1
//#define ID_RGB_GREEN                82
#define ID_RGB_GREEN_SLIDER         2
//#define ID_RGB_BLUE                 84
#define ID_RGB_BLUE_SLIDER          3
//#define ID_RGB_INTENSITY            86
//#define ID_RGB_INTENSITY_SLIDER     87

/****************************************************************************
  Function:
    void  AdjustColorPWM( int sliderID )

  Description:
    This function should be called when one of the color saturation sliders
    changes.  This routine will determine the new PWM output for the changed
    slider and set the PWM accordingly.

  Precondition:
    ShowScreenRGB() must be called prior to calling this function.

  Parameters:
    WORD sliderID   - ID of the slider to change
    SLIDER *pSlider - Pointer to the slider being changed.

  Returns:
    None

  Remarks:
    The PWM's run from 0 to 255, to match the saturation value.  The sliders
    run from 0 to 254.  The value 255 is omitted, because the value written
    to the PWM for the indicated saturation is the inverse of the slider
    value, and 0 (the inverse of 255) cannotbe written to the PWM.

    When the saturation level is 0, the PWM is turned off, and the pins are
    set to inputs to allow the LED's input to reach ~3.3V.  When the
    saturation level is greater than 0, the PWM is turned on if it was
    previously off, and the converted value is written to the PWM.
  ***************************************************************************/

void  AdjustColorPWM( int sliderID, int saturation )
{
    //WORD    saturation;

   // saturation  = SldGetPos( (SLIDER*)GOLFindObject( sliderID ) );

    if (sliderID == ID_RGB_RED_SLIDER) // Red slider changed
    {
        RGBSetRed( saturation );
    }

    else if (sliderID == ID_RGB_GREEN_SLIDER) // Green slider changed
    {
        RGBSetGreen( saturation );
    }

    else if (sliderID == ID_RGB_BLUE_SLIDER) // Blue slider changed
    {
        RGBSetBlue( saturation );
    }
}



  




/****************************************************************************
  Function:
    void RGBSetBlue( int saturation )

  Description:
    This function sets the blue RGB LED channel to the indicated saturation
    level.  If the saturation level is 0, then OxRS must be set to 0x100 to
    completely turn off that component of the color.

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    None
  ***************************************************************************/

void RGBSetBlue( int saturation )
{
    if (saturation == 0)
    {
        OC2RS = 0x100;
    }
    else
    {
        OC2RS = CONVERT_TO_COLOR( saturation );
    }
}


/****************************************************************************
  Function:
    void RGBSetGreen( int saturation )

  Description:
    This function sets the green RGB LED channel to the indicated saturation
    level.  If the saturation level is 0, then OxRS must be set to 0x100 to
    completely turn off that component of the color.

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    None
  ***************************************************************************/

void RGBSetGreen( int saturation )
{
    if (saturation == 0)
    {
        OC3RS = 0x100;
    }
    else
    {
        OC3RS = CONVERT_TO_COLOR( saturation );
    }
}


/****************************************************************************
  Function:
    void RGBSetRed( WORD saturation )

  Description:
    This function sets the red RGB LED channel to the indicated saturation
    level.  If the saturation level is 0, then OxRS must be set to 0x100 to
    completely turn off that component of the color.

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    None
  ***************************************************************************/

void RGBSetRed( int saturation )
{
    if (saturation == 0)
    {
        OC1RS = 0x100;
    }
    else
    {
        OC1RS = CONVERT_TO_COLOR( saturation );
    }
}


/****************************************************************************
  Function:
    void RGBMapColorPins( void )

  Description:
    This function sets up the Peripheral Pin Select to configure the three
    color control PWM's to the correct pins as per the schematic.

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    None
  ***************************************************************************/

void RGBMapColorPins( void )
{
    // Configure the LED Red Control, pins 31 (RP10) and 32 (RP17), for Output Compare 1 (function 18)
//    RPOR5bits.RP10R  = 18;
//    RPOR8bits.RP17R  = 18;
	
RPOR6bits.RP12R  = 18;   //RB12
    // Configure the LED Green Control, pins 6 (RP19) and 8 (RP27), for Output Compare 2 (function 19)
//    RPOR9bits.RP19R  = 19;
//    RPOR13bits.RP27R = 19;
RPOR6bits.RP13R = 19;  //RB13
    // Configure the LED Blue Control, pins 4 (RP21) and 5 (RP26), for Output Compare 3 (function 20)
//    RPOR10bits.RP21R = 20;
//    RPOR13bits.RP26R = 20;
 RPOR7bits.RP15R = 20;  //RB15
//
//    RPOR4            = 0;   // AN8 and AN9 PPS, leave analog
}


/****************************************************************************
  Function:
    void RGBTurnOffLED( void )

  Description:
    This function turns off the LED by turning off the timers, turning off
    the PWM's, and setting the port pins to inputs.  It should be called at
    the completion of the RGB LED demo.

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    None
  ***************************************************************************/

void RGBTurnOffLED( void )
{
    T2CON               = 0x0000;
    OC1CON             = PWM_OFF;
    OC2CON             = PWM_OFF;
    OC3CON             = PWM_OFF;

    TRISBbits.TRISB12    = TRIS_INPUT;
    TRISBbits.TRISB13    = TRIS_INPUT;
    TRISBbits.TRISB15    = TRIS_INPUT;
//    TRISGbits.TRISG9    = TRIS_INPUT;
//    TRISGbits.TRISG6    = TRIS_INPUT;
//    TRISGbits.TRISG7    = TRIS_INPUT;
}


/****************************************************************************
  Function:
    void RGBTurnOnLED( void )

  Description:
    This function turns on the LED by turning on the timers, turning on
    the PWM's, and setting the port pins to outputs.  It should be called at
    the beginning of the RGB LED demo.

  Precondition:
    None

  Parameters:
    None

  Returns:
    None

  Remarks:
    None
  ***************************************************************************/

void RGBTurnOnLED( int initialRed, int initialGreen, int initialBlue )
{
    // Initialize the timer for the PWM's.  We'll select a period value that
    // makes it simple to align the saturation values.
    T2CON                           = 0x0030;
    PR2                             = 0x00FF;

    // Initialize the PWM's
    RGBSetRed( initialRed );
    OC1R                            = 0;
   // OC1CON2                         = PWM_CONFIGURATION_2;
    OC1CON                         = PWM_CONFIGURATION_1;
    RGBSetGreen( initialGreen );
    OC2R                            = 0;
 //   OC2CON2                         = PWM_CONFIGURATION_2;
    OC2CON                         = PWM_CONFIGURATION_1;
    RGBSetBlue( initialBlue );
    OC3R                            = 0;
  //  OC3CON2                         = PWM_CONFIGURATION_2;
    OC3CON                        = PWM_CONFIGURATION_1;

    // Configure the PWM pins for output.
    TRISBbits.TRISB12                = TRIS_OUTPUT;   // Red
   // TRISFbits.TRISF5                = TRIS_OUTPUT;
    TRISBbits.TRISB13                = TRIS_OUTPUT;   // Green
  //  TRISGbits.TRISG9                = TRIS_OUTPUT;
    TRISBbits.TRISB15                = TRIS_OUTPUT;   // Blue
 //   TRISGbits.TRISG7                = TRIS_OUTPUT;

    // Enable open drain.
  //  ODCF                            = 0x0030;       // Red
   // ODCG                            = 0x03C0;       // Green and Blue

    // Turn on the timer.
    T2CON                           |= 0x8000;
}





  